@@ -12,6 +12,7 @@ from pptx.enum.text import (
     MSO_AUTO_SIZE, MSO_TEXT_UNDERLINE_TYPE, MSO_VERTICAL_ANCHOR,
     PP_PARAGRAPH_ALIGNMENT
 )
+from pptx.exc import InvalidXmlError
 from pptx.oxml import parse_xml
 from pptx.oxml.dml.fill import CT_GradientFillProperties
 from pptx.oxml.ns import nsdecls
@@ -28,30 +29,36 @@ from pptx.util import Emu, Length
 
 
 class CT_RegularTextRun(BaseOxmlElement):
-    """
-    Custom element class for <a:r> elements.
-    """
+    """`a:r` custom element class"""
+
     rPr = ZeroOrOne('a:rPr', successors=('a:t',))
     t = OneAndOnlyOne('a:t')
 
     @property
     def text(self):
-        """
-        The text of the ``<a:t>`` child element.
-        """
+        """(unicode) str containing text of (required) `a:t` child"""
         text = self.t.text
         # t.text is None when t element is empty, e.g. '<a:t/>'
         return to_unicode(text) if text is not None else u''
 
 
 class CT_TextBody(BaseOxmlElement):
+    """`p:txBody` custom element class.
+
+    Also used for `c:txPr` in charts and perhaps other elements.
     """
-    ``<p:txBody>`` custom element class, also used for ``<c:txPr>`` in
-    charts and perhaps other elements.
-    """
+
     bodyPr = OneAndOnlyOne('a:bodyPr')
     p = OneOrMore('a:p')
 
+    def clear_content(self):
+        """Remove all `a:p` children, but leave any others.
+
+        cf. lxml `_Element.clear()` method which removes all children.
+        """
+        for p in self.p_lst:
+            self.remove(p)
+
     @property
     def defRPr(self):
         """
@@ -65,6 +72,20 @@ class CT_TextBody(BaseOxmlElement):
         defRPr = pPr.get_or_add_defRPr()
         return defRPr
 
+    @property
+    def is_empty(self):
+        """True if only a single empty `a:p` element is present."""
+        ps = self.p_lst
+        if len(ps) > 1:
+            return False
+
+        if not ps:
+            raise InvalidXmlError('p:txBody must have at least one a:p')
+
+        if ps[0].text != '':
+            return False
+        return True
+
     @classmethod
     def new(cls):
         """
@@ -113,6 +134,16 @@ class CT_TextBody(BaseOxmlElement):
         txPr = parse_xml(xml)
         return txPr
 
+    def unclear_content(self):
+        """Ensure p:txBody has at least one a:p child.
+
+        Intuitively, reverse a ".clear_content()" operation to minimum
+        conformance with spec (single empty paragraph).
+        """
+        if len(self.p_lst) > 0:
+            return
+        self.add_p()
+
     @classmethod
     def _a_txBody_tmpl(cls):
         return (
@@ -283,9 +314,8 @@ class CT_TextNormalAutofit(BaseOxmlElement):
 
 
 class CT_TextParagraph(BaseOxmlElement):
-    """
-    <a:p> custom element class
-    """
+    """`a:p` custom element class"""
+
     pPr = ZeroOrOne('a:pPr', successors=(
         'a:r', 'a:br', 'a:fld', 'a:endParaRPr'
     ))
@@ -309,11 +339,18 @@ class CT_TextParagraph(BaseOxmlElement):
         return r
 
     def append_text(self, text):
+        """Append `a:r` and `a:br` elements to *p* based on *text*.
+
+        Any `\n` characters in *text* delimit `a:r` (run) elements and
+        themselves are translated to `a:br` (line-break) elements.
         """
-        Add *text* at the end of this paragraph element, translating line
-        feed characters ('\n') into ``<a:br>`` elements.
-        """
-        _ParagraphTextAppender.append_to_p_from_text(self, text)
+        for idx, r_str in enumerate(text.split('\n')):
+            # ---breaks are only added *between* items, not at start---
+            if idx > 0:
+                self.add_br()
+            # ---runs that would be empty are not added---
+            if r_str:
+                self.add_r(r_str)
 
     @property
     def content_children(self):
@@ -321,8 +358,14 @@ class CT_TextParagraph(BaseOxmlElement):
         A sequence containing the text-container child elements of this
         ``<a:p>`` element, i.e. (a:r|a:br|a:fld).
         """
-        text_types = (CT_RegularTextRun, CT_TextLineBreak, CT_TextField)
-        return tuple(elm for elm in self if isinstance(elm, text_types))
+        text_types = {CT_RegularTextRun, CT_TextLineBreak, CT_TextField}
+        return tuple(elm for elm in self if type(elm) in text_types)
+
+    @property
+    def text(self):
+        """str text contained in this paragraph."""
+        # ---note this shadows the lxml _Element.text---
+        return ''.join([child.text for child in self.content_children])
 
     def _new_r(self):
         r_xml = '<a:r %s><a:t/></a:r>' % nsdecls('a')
@@ -455,55 +498,3 @@ class CT_TextSpacingPoint(BaseOxmlElement):
     attribute.
     """
     val = RequiredAttribute('val', ST_TextSpacingPoint)
-
-
-class _ParagraphTextAppender(object):
-    """
-    Service object that knows how to translate a Python string into run and
-    line break elements appended to a specified ``<a:p>`` element. Contiguous
-    sequences of regular characters are appended in a single ``<a:r>``
-    element. A newline character ('\n') causes a ``<a:br>`` element to be
-    appended.
-    """
-    def __init__(self, p):
-        self._p = p
-        self._bfr = []
-
-    @classmethod
-    def append_to_p_from_text(cls, p, text):
-        """
-        Create a "one-shot" ``_ParagraphTextAppender`` instance and use it to
-        append ``<a:r>`` and ``<a:br>`` elements to *p* that correspond to
-        the contents of *text*.
-        """
-        appender = cls(p)
-        appender._add_text(text)
-
-    def _add_text(self, text):
-        """
-        Append the paragraph content elements corresponding to *text* to the
-        ``<a:p>`` element of this instance.
-        """
-        for char in text:
-            self._add_char(char)
-        self._flush()
-
-    def _add_char(self, char):
-        """
-        Process the next character of input through the translation finite
-        state maching (FSM). There are two possible states, buffer pending
-        and not pending, but those are hidden behind the ``.flush()`` method
-        which must be called at the end of text to ensure any pending
-        ``<a:r>`` element is written.
-        """
-        if char == '\n':
-            self._flush()
-            self._p.add_br()
-        else:
-            self._bfr.append(char)
-
-    def _flush(self):
-        text = ''.join(self._bfr)
-        if text:
-            self._p.add_r(text)
-        del self._bfr[:]
